#!/home/lukas/dev/COVID-19/csse_covid_19_data/csse_covid_19_daily_reports/venv/bin/python3
import csv
from collections import OrderedDict

# external deps - plotly
from plotly import offline
import plotly.graph_objects as go

# assume per_country csvs generated by per_country

rate_cutoff = 1.5  # do not show R7 until daily 7 day moving average new cases per million reach at least this value

##### copy over from per_country.py
##### dont forget to copy DISPLAY_NAME over as well
countries = [
    "USA",  # states in us_countries
    "Canada",  # provinces set in canada_countries
    "EU",  # eu countries in eu_countries
    # Europe
    "UK", "Serbia", "Albania", "Russia", "Ukraine", "Belarus", "Switzerland", "Moldova",
    "Bosnia and Herzegovina", "Iceland", "Faroe Islands",
    # APAC
    "Guam (US)", "Singapore", "Taiwan", "New Zealand", "Japan", "South Korea", "Cambodia",
    "Vietnam", "Malaysia", "Thailand", "Laos", "Philippines", "Indonesia", "Papua New Guinea", "Australia",
    # Africa
    "Seychelles", "Egypt", "Senegal", "Burkina Faso", "Nigeria", "Morocco",
    "Tunisia", "Mali", "Kenya", "South Africa", "Isle of Man",
    # Latin America
    "Peru", "Mexico", "Cuba", "Guatemala", "Costa Rica", "Brazil", "Bolivia", "Chile",
    "Argentina", "Uruguay", "Paraguay", "Colombia",
    # Middle East and Central Asia
    "Kazakhstan", "Uzbekistan", "Kyrgyzstan", "Mongolia", "Iraq", "Iran", "Saudi Arabia", "Jordan", "Israel", "Turkey",
    # Others
    "India", "Pakistan", "Nepal", "Falkland Islands", "Greenland",
]

eu_countries = [
    "Czechia", "Netherlands", "Italy", "Slovakia", "Ireland", "Denmark", "Spain", "Luxembourg", "Slovenia",
    "Greece", "Sweden", "Austria", "Belgium", "Bulgaria", "Croatia", "Cyprus", "Estonia", "Finland", "France",
    "Germany", "Hungary", "Latvia", "Lithuania", "Malta", "Poland", "Portugal", "Romania",
]
countries.extend(eu_countries)

us_countries = ["Alabama", "Alaska", "Arizona", "Arkansas", "California", "Colorado", "Connecticut", "Delaware",
    "Florida", "Georgia", "Hawaii", "Idaho", "Illinois", "Indiana", "Iowa", "Kansas", "Kentucky", "Louisiana", "Maine",
    "Maryland", "Massachusetts", "Michigan", "Minnesota", "Mississippi", "Missouri", "Montana", "Nebraska", "Nevada", "New Hampshire",
    "New Jersey", "New Mexico", "New York", "North Carolina", "North Dakota", "Ohio", "Oklahoma", "Oregon", "Pennsylvania",
    "Rhode Island", "South Carolina", "South Dakota", "Tennessee", "Texas", "Utah", "Vermont", "Virginia", "Washington", "West Virginia",
    "Wisconsin", "Wyoming"]
countries.extend(us_countries)

canada_countries = ["Alberta", "British Columbia", "Manitoba", "New Brunswick", "Newfoundland and Labrador", "Northwest Territories",
    "Nova Scotia", "Ontario", "Prince Edward Island", "Quebec", "Saskatchewan", "Yukon"]
countries.extend(canada_countries)

italy_regions = [
"Abruzzo (Italy region)", "Basilicata (Italy region)", 
"P.A. Bolzano (Italy region)",  # south tyrol
"Calabria (Italy region)", "Campania (Italy region)", "Emilia-Romagna (Italy region)", "Friuli Venezia Giulia (Italy region)",
"Lazio (Italy region)", "Liguria (Italy region)", "Lombardia (Italy region)", "Marche (Italy region)", "Molise (Italy region)",
"Piemonte (Italy region)", "Puglia (Italy region)", "Sardegna (Italy region)", "Sicilia (Italy region)", "Toscana (Italy region)",
"P.A. Trento (Italy region)", "Umbria (Italy region)", "Valle d'Aosta (Italy region)", "Veneto (Italy region)",
]
countries.extend(italy_regions)

DISPLAY_NAME = {
    "Czechia": "Czech Republic",
    "Guam (US)": "Guam",
}
for c in us_countries:
    if c == "New York":
        DISPLAY_NAME[c] = "New York (state) (US)"
    else:
        DISPLAY_NAME[c] = c + " (US)"
for c in canada_countries:
    DISPLAY_NAME[c] = c + " (Canada)"
##### end

#### copy from per_country_index.py
annotated = [
    "Singapore", "Czechia", "Netherlands", "Italy", "Taiwan", "New Zealand", "Slovakia", "Japan", "South Korea", "Ireland",
    "Denmark", "Serbia", "Vietnam", "Albania", "Spain", "Seychelles", "Luxembourg", "Slovenia", "Greece", "Guam (US)",
    "Sweden", "Russia", "Peru", "UK", "USA", "California", "Virginia", "New York", "Iowa", "Montana", "Nebraska", "Wisconsin",
]
annotated = [DISPLAY_NAME[c] if c in DISPLAY_NAME else c for c in annotated]
annotated.sort()
#### end
eu_countries.sort()
us_countries.sort()
italy_regions.sort()
countries.sort()
all_nobreakdown_countries = list(set(countries) - set(us_countries) - set(canada_countries) - set(italy_regions))
all_nobreakdown_countries.sort()

# top of 20'
northern_hemisphere = [
    "USA", "Canada", "EU", "UK", "Serbia", "Albania", "Russia", "Ukraine", "Belarus",
    "Czechia", "Netherlands", "Italy", "Slovakia", "Ireland", "Denmark", "Spain", "Slovenia", "Greece", "Sweden", 
    "Switzerland", "Moldova", "Bosnia and Herzegovina", "Iceland", "Faroe Islands",
    "Taiwan", "Japan", "South Korea", "Morocco", "Tunisia", "Isle of Man", "Kazakhstan",
    "Uzbekistan", "Kyrgyzstan", "Mongolia", "Iraq", "Iran", "Jordan", "Israel", "Turkey", "Nepal"
]
northern_hemisphere.sort()

# south of 20'
southern_hemisphere = [
    "New Zealand", "Australia", "South Africa", "Peru", "Brazil", "Bolivia", "Chile",
    "Argentina", "Uruguay", "Paraguay",
]
southern_hemisphere.sort()

# in some countries 21 day CFD can be >100% (this might be legit, more people died in last 21 days
# than new cases), or it might be <0% (faking the numbers by remove a huge number of deaths at once),
# remove the countries/regions with extreme 21 day CFD values to make the graphs readable
extreme_cfd_countries = [
    "Abruzzo (Italy region)", "Australia", 
    "Basilicata (Italy region)", "P.A. Bolzano (Italy region)", "Campania (Italy region)", "Friuli Venezia Giulia (Italy region)",
    "Lazio (Italy region)", "Liguria (Italy region)", "Marche (Italy region)", "P.A. Trento (Italy region)", "Piemonte (Italy region)",
    "Puglia (Italy region)", "Sardegna (Italy region)", "Sicilia (Italy region)", "Puglia (Italy region)", "Veneto (Italy region)",
    "Sardegna (Italy region)", "Sicilia (Italy region)",  "Toscana (Italy region)",  "Umbria (Italy region)", "Valle d'Aosta (Italy region)", 
    "Croatia", "France", "Isle of Man", "Marche", "Nebraska (US)", "Washington (US)", "Montana (US)",
    "New Jersey (US)", "New Zealand", "Nova Scotia (Canada)", "Prince Edward Island (Canada)",
    "Slovenia", "Spain", "Taiwan", "Wisconsin (US)", "Philippines",
]

# list of countries where the latest 7 day average of new cases is >= 0.8 * the max value
raising_countries = []
for country in countries:
    display_name = country
    if country in DISPLAY_NAME:
        display_name = DISPLAY_NAME[country]
    f = open("per_country/"+display_name+".csv")
    r = csv.reader(f)
    header = next(r)
    d_max = None
    d_last = None
    for line in r:
        d = dict(zip(header, line))
        try:
            d = float(d['new_cases_7d_mavg'])
        except:
            continue
        d_last = d
        if d_max is None or d_last > d_max:
            d_max = d_last
    if d_last is not None:
        if d_last >= d_max * 0.8:
            raising_countries.append(country)
#

comparisons = OrderedDict()
comparisons["EU"] = {
    "title": "EU",
    "countries": [DISPLAY_NAME[c] if c in DISPLAY_NAME else c for c in eu_countries],
    "file_base": "compare_eu",
    "first_date": "2020-03-15",
    "rate_max": 6,
}

comparisons["US"] = {
    "title": "US",
    "countries": [DISPLAY_NAME[c] if c in DISPLAY_NAME else c for c in us_countries],
    "file_base": "compare_us",
    "first_date": "2020-03-15",
    "rate_max": 6,
}

comparisons["annotated"] = {
    "title": "(for annotated graphs)",
    "countries": [DISPLAY_NAME[c] if c in DISPLAY_NAME else c for c in annotated],
    "file_base": "compare_annotated",
    "first_date": "2020-03-15",
    "rate_max": 6,
}

comparisons["All"] = {
    "title": "all countries",
    "countries": [DISPLAY_NAME[c] if c in DISPLAY_NAME else c for c in countries],
    "file_base": "compare_all",
    "first_date": "2020-03-15",
    "rate_max": 6,
}

comparisons["Italy regions"] = {
    "title": "Italy regions",
    "countries": [DISPLAY_NAME[c] if c in DISPLAY_NAME else c for c in italy_regions],
    "file_base": "italy_regions",
    "first_date": "2020-03-01",
    "rate_max": 6,
}

comparisons["All_nobreakdown"] = {
    "title": "all countries",
    "countries": [DISPLAY_NAME[c] if c in DISPLAY_NAME else c for c in all_nobreakdown_countries],
    "file_base": "compare_all_nobreakdown",
    "first_date": "2020-03-15",
    "rate_max": 6,
}

comparisons["Raising"] = {
    "title": "countries w/ currently raising no. of cases",
    "countries": [DISPLAY_NAME[c] if c in DISPLAY_NAME else c for c in raising_countries],
    "file_base": "compare_raising",
    "first_date": "2020-03-15",
    "rate_max": 6,
}

comparisons["northern_hemisphere"] = {
    "title": "northern hemisphere countries",
    "countries": [DISPLAY_NAME[c] if c in DISPLAY_NAME else c for c in northern_hemisphere],
    "file_base": "north",
    "first_date": "2020-03-15",
    "rate_max": 6,
}

comparisons["southern_hemisphere"] = {
    "title": "southern hemisphere countries",
    "countries": [DISPLAY_NAME[c] if c in DISPLAY_NAME else c for c in southern_hemisphere],
    "file_base": "south",
    "first_date": "2020-03-15",
    "rate_max": 6,
}


def generate_graph(countries, out_file_base, first_date = None, title=None, rate_max=5):
    # graph
    print(f"generating graphs ({out_file_base})")

    if title is None:
        title = out_file_base

    countries_dict = {}
    # key:
    #   country
    # vals:
    #   {
    #       first_day = (date of first confirmed case in the country)
    #       by_date = {
    #           "date": 
    #            {
    #               "rate": float (or None)   # R7
    #               "cases": float (or None)  # 7 day moving average of new cases per million
    #               "cases_reported": float (or None)  # 7 day moving average of new cases (absolute)
    #               "cases_total": float # total number of cases per million
    #               "cases_total_reported": int # total number of cases
    #               "mortality": float (or None)  # 7 day moving average of death toll per million
    #               "mortality_reported": float (or None)  # 7 day moving average of death toll
    #               "mortality_total": float  # total death toll per million
    #               "mortality_total_reported": int # total death toll
    #               "recovered": float (or None)  # 7 day moving average of recovered per million
    #               "recovered_reported": float (or None)  # 7 day moving average of recovered
    #               "recovered_total": float  # total recovered per million
    #               "recovered_total_reported": int # total recovered
    #               "cfr": float # cfr in percents
    #               "deaths_population": float # death toll as % of population 
    #            }
    #       }
    #       by_day = {
    #           day:  # n-th day since first case, indexed from 1, int
    #            {
    #               "rate": float (or None)   # R7
    #               "cases": float (or None)  # 7 day moving average of new cases per million
    #               "cases_reported": float (or None)  # 7 day moving average of new cases (absolute)
    #               "cases_total": float # total number of cases per million
    #               "cases_total_reported": int # total number of cases
    #               "mortality": float (or None)  # 7 day moving average of new cases per million
    #               "mortality_reported": float (or None)  # 7 day moving average of death toll
    #               "mortality_total": float  # total death toll per million
    #               "mortality_total_reported": int # total death toll
    #               "recovered": float (or None)  # 7 day moving average of recovered per million
    #               "recovered_reported": float (or None)  # 7 day moving average of recovered
    #               "recovered_total": float  # total recovered per million
    #               "recovered_total_reported": int # total recovered
    #               "cfr": float # cfr in percents
    #               "deaths_population": float # death toll as % of population 
    #            }
    #       }
    #   }

    def get_date_val(country, key, date, default_value=None):
        d = countries_dict[country]["by_date"]
        if date not in d:
            return default_value
        if key not in d[date]:
            return default_value
        return d[date][key]

    def get_day_val(country, key, day, default_value=None):
        d = countries_dict[country]["by_day"]
        if day not in d:
            return default_value
        if key not in d[day]:
            return default_value
        return d[day][key]

    dates = set()
    for country in countries:
        f = open(f"per_country/{country}.csv")
        r = csv.reader(f)
        header = next(r)
        for line in r:
            d = dict(zip(header, line))
            dates.add(d['date'])
    # YYYY-MM-DD so sort should work fine
    # all dates
    dates = list(dates)
    dates.sort()
    days = list(range(1, len(dates)+1))

    for country in countries:
        rate_cutoff_reached = False
        f = open(f"per_country/{country}.csv")
        r = csv.reader(f)
        header = next(r)
        first_line = True
        for line in r:
            d = dict(zip(header, line))
            if first_line:
                countries_dict[country] = {
                    "first_day": d['date'],
                    "by_date": OrderedDict(),
                    "by_day": OrderedDict(),
                }
                first_line = False

            values = {}
            if d['new_cases_per_million_7d_mavg']:
                values["cases"] = float(d['new_cases_per_million_7d_mavg'])
                if values["cases"] > rate_cutoff:
                    rate_cutoff_reached = True
            if d['rate']:
                if rate_cutoff is not None:
                    if rate_cutoff_reached:
                        values["rate"] = float(d['rate'])
                else:
                    values["rate"] = float(d['rate'])
            if d['new_cases_7d_mavg']:
                values["cases_reported"] = float(d["new_cases_7d_mavg"])
            if d['total_cases_per_million']:
                values["cases_total"] = float(d['total_cases_per_million'])
            if d['total_cases']:
                values["cases_total_reported"] = int(d['total_cases'])
            if d['new_deaths_per_million_7d_mavg']:
                values["mortality"] = float(d['new_deaths_per_million_7d_mavg'])
            if d['new_deaths_7d_mavg']:
                values["mortality_reported"] = float(d['new_deaths_7d_mavg'])
            if d['total_deaths_per_million']:
                values["mortality_total"] = float(d['total_deaths_per_million'])
            if d['total_deaths']:
                values["mortality_total_reported"] = float(d['total_deaths'])
            if d['new_recovered_per_million_7d_mavg']:
                values["recovered"] = float(d['new_recovered_per_million_7d_mavg'])
            if d['new_recovered_7d_mavg']:
                values["recovered_reported"] = float(d['new_recovered_7d_mavg'])
            if d['total_recovered_per_million']:
                values["recovered_total"] = float(d['total_recovered_per_million'])
            if d['total_recovered']:
                values["recovered_total_reported"] = d['total_recovered']
            values['cfr'] = float(d['cfr']) * 100
            values['cfr_last_21d'] = float(d['cfr_last_21d']) * 100
            values['deaths_population'] = float(d['total_deaths_population_ratio']) * 100

            countries_dict[country]["by_date"][d['date']] = values
            countries_dict[country]["by_day"][int(d['day'])] = values

    figs = [
            # (plotly_graph_figure, key_in_countries_dict, by date (False) or by day (True), 
            #   special_treatment ("moving_average", "rate", "remove_countries_with_extreme_cfd", None), out_file_basename)
            # rate
            (go.Figure(), "rate", False, "rate", f"{out_file_base}_rate"),
            (go.Figure(), "rate", True, "rate", f"{out_file_base}_rate_first_day"),
            # confirmed cases
            (go.Figure(), "cases", False, "moving_average", f"{out_file_base}_cases"),
            (go.Figure(), "cases", True, "moving_average", f"{out_file_base}_cases_first_day"),
            (go.Figure(), "cases_total", False, None, f"{out_file_base}_cases_total"),
            (go.Figure(), "cases_total", True, None, f"{out_file_base}_cases_total_first_day"),
            (go.Figure(), "cases_reported", False, "moving_average", f"{out_file_base}_cases_reported"),
            (go.Figure(), "cases_reported", True, "moving_average", f"{out_file_base}_cases_reported_first_day"),
            (go.Figure(), "cases_total_reported", False, None, f"{out_file_base}_cases_reported_total"),
            (go.Figure(), "cases_total_reported", True, None, f"{out_file_base}_cases_reported_total_first_day"),
            # mortality
            (go.Figure(), "mortality", False, "moving_average", f"{out_file_base}_mortality"),
            (go.Figure(), "mortality", True, "moving_average", f"{out_file_base}_mortality_first_day"),
            (go.Figure(), "mortality_total", False, False, f"{out_file_base}_mortality_total"),
            (go.Figure(), "mortality_total", True, False, f"{out_file_base}_mortality_total_first_day"),
            (go.Figure(), "mortality_reported", False, "moving_average", f"{out_file_base}_mortality_reported"),
            (go.Figure(), "mortality_reported", True, "moving_average", f"{out_file_base}_mortality_reported_first_day"),
            (go.Figure(), "mortality_total_reported", False, False, f"{out_file_base}_mortality_reported_total"),
            (go.Figure(), "mortality_total_reported", True, False, f"{out_file_base}_mortality_reported_total_first_day"),
            # recovered
            (go.Figure(), "recovered", False, "moving_average", f"{out_file_base}_recovered"),
            (go.Figure(), "recovered", True, "moving_average", f"{out_file_base}_recovered_first_day"),
            (go.Figure(), "recovered_total", False, False, f"{out_file_base}_recovered_total"),
            (go.Figure(), "recovered_total", True, False, f"{out_file_base}_recovered_total_first_day"),
            (go.Figure(), "recovered_reported", False, "moving_average", f"{out_file_base}_recovered_reported"),
            (go.Figure(), "recovered_reported", True, "moving_average", f"{out_file_base}_recovered_reported_first_day"),
            (go.Figure(), "recovered_total_reported", False, False, f"{out_file_base}_recovered_reported_total"),
            (go.Figure(), "recovered_total_reported", True, False, f"{out_file_base}_recovered_reported_total_first_day"),
            # cfr
            (go.Figure(), "cfr", False, False, f"{out_file_base}_cfr"),
            (go.Figure(), "cfr", True, False, f"{out_file_base}_cfr_first_day"),
            # cfr last 21d
            (go.Figure(), "cfr_last_21d", False, "remove_countries_with_extreme_cfd",f"{out_file_base}_cfr_last_21d"),
            (go.Figure(), "cfr_last_21d", True, "remove_countries_with_extreme_cfd",f"{out_file_base}_cfr_last_21d_first_day"),
            # deaths in % of population
            (go.Figure(), "deaths_population", False, False, f"{out_file_base}_deaths_population"),
            (go.Figure(), "deaths_population", True, False, f"{out_file_base}_deaths_population_first_day"),
    ]

    skipped_country = []
    for fig, key, by_day, special_treatment, _  in figs:
        if by_day:
            xvals = days
            xvals_text = [str(date_) for date_ in xvals]
            xtitle = "Number of days since first reported case in the country"
            xrange_ = [xvals[0], len(xvals)]
            if special_treatment == "moving_average":
                xtitle += '<span style="font-size: 12px"><br>'\
                    "(note: starting on day 4 because we are calculating 7-day moving average)</span>"
                xrange_ = [xvals[3], len(xvals)-3]
            elif special_treatment == "rate":
                xrange_ = [xvals[3], len(xvals)-10]
        else:
            xvals = dates
            xvals_text = [date_.replace("-", "/") for date_ in xvals]
            xtitle = "date"
            if first_date is not None:
                xrange_= [first_date, xvals[-1]]
            if special_treatment == "moving_average":
                xrange_[1] = xvals[-4]
            elif special_treatment == "rate":
                xrange_[1] = xvals[-11]

        fig.update_xaxes(
            title= {'text': xtitle, 'font': {'size': 20}},
            tickvals = xvals,
            ticktext = xvals_text,
            tickangle = 80,
            range = xrange_,
        )

        for i, country in enumerate(countries):
            if special_treatment == "remove_countries_with_extreme_cfd":
                if country in extreme_cfd_countries:
                    continue
            if country not in countries_dict:
                if country not in skipped_country:
                    print("skipping " + country + " in compare graph - missing record in countries_dict")
                    skipped_country.append(country)
                continue
            if by_day:
                yvals = [get_day_val(country, key, x, None) for x in xvals]
            else:
                yvals = [get_date_val(country, key, x, None) for x in xvals]

            line_i = i % 50
            line = {'dash':'solid', 'width': 2}
            if line_i >= 10:
                line = {'dash':'dot', 'width': 2}
            if line_i >= 20:
                line = {'dash':'dash', 'width': 2}
            if line_i >= 30:
                line = {'dash':'longdash', 'width': 2}
            if line_i >= 40:
                line = {'dash':'dashdot', 'width': 2}
            if line_i >= 50:
                line = {'dash':'longdashdot', 'width': 2}

            fig.add_trace(
                go.Scatter(
                    mode='lines',
                    x=xvals,
                    y=yvals,
                    name=country,
                    line=line,
                )
            )

        # layout, title, formatting etc.
        fig.update_layout(
            title = {
                'xanchor': 'center',
                'x': 0.5,
                'font': {'size': 30},
            },
            showlegend=True,
        )

        fig.update_yaxes(
            title = {
                'font': {'size': 18},
            },
            color="crimson",
            rangemode='tozero',
            showgrid=True,
        )
        if special_treatment is None or special_treatment == "moving_average" or special_treatment == "remove_countries_with_extreme_cfd":
            # some countries sometimes report negative values for new cases or deaths
            # that's fine and we properly show this seamingly nonsense values in
            # the individual graphs, not in comparisons though as it would "dilute"
            # the graph too much
            fig.update_yaxes(rangemode='nonnegative')

        if key == "rate":
            fig.update_layout(title_text = f'7-day rate (R7): {title}')
            fig.update_yaxes(
                title = {
                    'text': "(R)ate of change - 7-day rate <br>"
                            "calculated as today+7 days/today using moving average of daily new cases<br>"
                           f"(calculated only after the moving average has reached {rate_cutoff})",
                    'font' : {'size': 14},
                },
                # fixedrange=True,
                range=(0,rate_max+.05),
            )
        elif key == "cases":
            fig.update_layout(title_text = f'New cases {title} (per million)')
            fig.update_yaxes(title_text = "New cases per million<br>7-day moving average")
        elif key == "cases_total":
            fig.update_layout(title_text = f'Total confirmed cases {title} (per million)')
            fig.update_yaxes(title_text = "Total confirmed cases (per million)")
        elif key == "cases_reported":
            fig.update_layout(title_text = f'New cases {title}')
            fig.update_yaxes(title_text = "New cases<br>7-day moving average")
        elif key == "cases_total_reported":
            fig.update_layout(title_text = f'Total confirmed cases {title}')
            fig.update_yaxes(title_text = "Total confirmed cases")
        elif key == "mortality":
            fig.update_layout(title_text = f'Daily death toll {title} (per million)')
            fig.update_yaxes(title_text = "Daily death toll per million<br>7-day moving average")
        elif key == "mortality_total":
            fig.update_layout(title_text = f'Total death toll {title} (per million)')
            fig.update_yaxes(title_text = "total death toll per million")
        elif key == "mortality_reported":
            fig.update_layout(title_text = f'Daily death toll {title}')
            fig.update_yaxes(title_text = "Daily death toll<br>7-day moving average")
        elif key == "mortality_total_reported":
            fig.update_layout(title_text = f'Total death toll {title}')
            fig.update_yaxes(title_text = "Total death toll")
        elif key == "cfr":
            fig.update_layout(title_text = f'Case-fatality ratio (%) {title}')
            fig.update_yaxes(title_text = "CFR (%)")
        elif key == "cfr_last_21d":
            fig.update_layout(title_text = f'Last 21 days case-fatality ratio (%) {title}')
            fig.update_yaxes(title_text = "21 days CFR (%)")
        elif key == "deaths_population":
            fig.update_layout(title_text = f'Total death toll as percentage of population {title}')
            fig.update_yaxes(title_text = "Total death toll as percentage of population (%)")
        elif key == "recovered":
            fig.update_layout(title_text = f'Daily recovered {title} (per million)')
            fig.update_yaxes(title_text = "Daily recovered per million<br>7-day moving average")
        elif key == "recovered_total":
            fig.update_layout(title_text = f'Total recovered {title} (per million)')
            fig.update_yaxes(title_text = "total recovered per million")
        elif key == "recovered_reported":
            fig.update_layout(title_text = f'Daily recovered {title}')
            fig.update_yaxes(title_text = "Daily recovered<br>7-day moving average")
        elif key == "recovered_total_reported":
            fig.update_layout(title_text = f'Total recovered {title}')
            fig.update_yaxes(title_text = "Total recovered")

    for fig, _, _, _, out_file  in figs:
        offline.plot(fig, filename=f"per_country/{out_file}.html", auto_open=False)

for c in comparisons.values():
    generate_graph(c["countries"], c["file_base"], c["first_date"], c["title"], c["rate_max"])
